@@ -1,4 +1,5 @@
 from .geometry import Geometry
 from .material import Material
-from .shared import Solver
+from .solver import Solver
 from .plot import Plot
+from .user_custom import UserStructure
