@@ -1,5 +1,4 @@
 import numpy as np
-#from numpy import inf
 import sys
 from .utils import *
 
@@ -14,10 +13,9 @@ def main():
  tau = tmp[:,7]
  v   = tmp[:,8:]
  C   = tmp[:,6]
+ w   = tmp[:,5]
  kappa =  np.einsum('ki,kj,k,k',v,v,tau,C) 
 
- data = {'C':C,'tau':tau,'v':v,'kappa':kappa}
- 
-
+ data = {'C':C,'tau':tau,'v':v,'kappa':kappa,'f':w/2.0/np.pi}
 
  save_data('rta',data)   
