@@ -10,10 +10,10 @@ import scipy.sparse as sp
 import time
 import sys
 import scipy
-
-
+from matplotlib.pylab import *
+from cachetools import cached
+from cachetools.keys import hashkey
 comm = MPI.COMM_WORLD
-
       
 def get_m(Master,data):
     
@@ -48,6 +48,28 @@ def print_multiscale(argv,MM):
         print(colored(' -----------------------------------------------------------','green'),flush=True)
 
 
+@cached(cache={}, key=lambda data,indices:hashkey(indices))
+def compute_lu(A,indices):
+
+ return sp.linalg.splu(A)
+
+@cached(cache={}, key=lambda data,indices:hashkey(indices))
+def compute_spilu(A,indices):
+
+ return sp.linalg.spilu(A)
+
+
+
+def solve(argv,A,B,indices):
+
+    if not argv['keep_lu']:
+        X = sp.linalg.spsolve(A,B,use_umfpack=True)
+    else: 
+        X = compute_lu(A,indices).solve(B)
+
+    return X 
+
+
 
 def solve_rta(**argv):
 
@@ -64,7 +86,7 @@ def solve_rta(**argv):
        print(colored(' -----------------------------------------------------------','green'),flush=True)
 
     if len(mesh['db']) > 0:
-      if comm.rank == 0:  
+      if comm.rank == 0: 
        Gb   = np.einsum('mqj,jn->mqn',mat['sigma'],mesh['db'],optimize=True)
        Gbp2 = Gb.clip(min=0);
        with np.errstate(divide='ignore', invalid='ignore'):
@@ -83,6 +105,7 @@ def solve_rta(**argv):
     np.add.at(D.T,mesh['i'],Gp.T)
 
     DeltaT = fourier['temperature_fourier'].copy()
+    #DeltaT_initial = DeltaT.copy()
     #--------------------------
 
     #---boundary----------------------
@@ -94,11 +117,6 @@ def solve_rta(**argv):
      np.add.at(D.T,mesh['eb'],Gbp2.T)
      TB = DeltaT[mesh['eb']]
 
-     #for n,i in enumerate(mesh['eb']):
-     #    D[:, i]  += Gbp2[:,n]
-     #    TB[:,n]  = DeltaT[i] 
-
-
     #Periodic---
     sss = np.asarray(mesh['pp'][:,0],dtype=int)
     P = np.zeros((len(argv['rr']),n_elems))
@@ -113,37 +131,25 @@ def solve_rta(**argv):
     error = 1
     kk = 0
     kappa_tot = np.zeros(1)
+    kappaf_tot = np.zeros(1)
     MM = np.zeros(2)
     Mp = np.zeros(2)
     kappa,kappap = np.zeros((2,argv['n_serial'],argv['n_parallel']))
+    kappaf,kappap_f = np.zeros((2,argv['n_serial'],argv['n_parallel']))
+    kappa0,kappap_0 = np.zeros((2,argv['n_serial'],argv['n_parallel']))
+    kappab,kappap_b = np.zeros((2,argv['n_serial'],argv['n_parallel']))
     transitionp = np.zeros(argv['n_parallel'])
     transition = 1e4 * np.ones(argv['n_parallel'])
 
-
-    #T_old = np.tile(DeltaT,(argv['n_parallel'],argv['n_serial']))
+    mat['tc'] = mat['tc']/np.sum(mat['tc'])
 
     Master = sp.csc_matrix((np.arange(len(mesh['im']))+1,(mesh['im'],mesh['jm'])),shape=(n_elems,n_elems),dtype=np.float64)
     conversion = np.asarray(Master.data-1,np.int) 
 
     J = np.zeros((n_elems,argv['dim']))
 
-  
-    #DELTA---
-       
-    #----------------
-
-
-    cache = {}
+    a = time.time()
     while kk < argv['max_bte_iter'] and error > argv['max_bte_error']:
-
-        #for ll in mesh['periodic_sides']:
-        # (k1,k2) = mesh['side_elem_map_vec'][ll] 
-        # DT = abs(DeltaT[k1] - DeltaT[k2])
-        # if DT > 0.1:
-        #print(DT)
-
-
-        a = time.time()
         if argv['multiscale']: tf,tfg = solve_fourier(mat['mfp_average'],DeltaT,argv)
         #Multiscale scheme-----------------------------
         diffusive = 0
@@ -152,137 +158,124 @@ def solve_rta(**argv):
         TBp = np.zeros_like(TB)
         Jp = np.zeros_like(J)
         kappa_bal,kappa_balp = np.zeros((2,argv['n_parallel']))
-        #Sup,Supp   = np.zeros((2,len(mat['kappam'])))
-        #Supd,Supdp   = np.zeros((2,len(mat['kappam'])))
-        #Supb,Supbp   = np.zeros((2,len(mat['kappam'])))
 
-        RHS = -np.einsum('c,nc->nc',TB,Gbm2) if len(mesh['db']) > 0 else  np.zeros((argv['n_serial'],argv['n_parallel'],0)) 
+        RHS = -np.einsum('c,nc->nc',TB,Gbm2) if len(mesh['db']) > 0 else  np.zeros((argv['n_parallel'],0)) 
 
         for n,q in enumerate(argv['rr']):
            
            #Get ballistic kappa
            if argv['multiscale']:
-              kappa_fourier_m =  np.einsum('c,mc->m',mesh['kappa_mask'],tf) - np.einsum('c,m,i,mci->m',mesh['kappa_mask'],mfp,mat['VMFP'][q,0:argv['dim']],tfg)
-              #Supdp += np.einsum('m,mu->u',kappa_fourier_m,mat['suppression'][:,q,:])*1e9
+              kappap_f[:,q] = np.einsum('c,mc->m',mesh['kappa_mask'],tf) - np.einsum('c,m,i,mci->m',mesh['kappa_mask'],mfp,mat['VMFP'][q,0:argv['dim']],tfg)
 
-
-              B = DeltaT +  mfp[-1]*(P[n] +get_boundary(RHS[n],mesh['eb'],n_elems))
+              #------------------------------------------------------
+              B = DeltaT +  mfp[-1]*(P[n] + get_boundary(RHS[n],mesh['eb'],n_elems))
               A = get_m(Master,np.concatenate((mfp[-1]*Gm[n],mfp[-1]*D[n]+np.ones(n_elems)))[conversion])
-              if not argv['keep_lu']:
-                  X_bal = sp.linalg.spsolve(A,B,use_umfpack=True)
-              else: 
-                  X_bal = (lu[(-1,n)] if (-1,n) in lu.keys() else lu.setdefault((-1,n),sp.linalg.splu(A))).solve(B)
+              X_bal = solve(argv,A,B,(q,-1))
+              kappap_b[:,q] = np.dot(mesh['kappa_mask'],X_bal)
+              #-----------------------------------------------------
 
-              kappa_bal = np.dot(mesh['kappa_mask'],X_bal)
+              idx = np.argwhere(np.diff(np.sign(kappap_b[:,q] - kappap_f[:,q]))).flatten()+1
+              if len(idx) == 0:
+                 idx = argv['n_serial']-1
+              elif len(idx) > 1:
+                 idx = idx[-1] #take the last one to be on the safe side
+              else:   
+                 idx = idx[0]
 
-              idx = np.argwhere(np.diff(np.sign(kappa_bal*np.ones(argv['n_serial']) - kappa_fourier_m))).flatten()
-              if len(idx) == 0: idx = [argv['n_serial']-1]
-           else: idx = [argv['n_serial']-1]
+              idx = min([idx,argv['n_serial']])
 
-           fourier = False
-           for m in range(argv['n_serial'])[idx[0]::-1]:
-                 
+           else: idx = argv['n_serial']-1
+
+           for m in range(argv['n_serial'])[idx::-1]:
+               
+                 #-------------------------
                  B = DeltaT +  mfp[m]*(P[n] +get_boundary(RHS[n],mesh['eb'],n_elems))
                  A = get_m(Master,np.concatenate((mfp[m]*Gm[n],mfp[m]*D[n]+np.ones(n_elems)))[conversion])
-                 if not argv['keep_lu']:
-                  X = sp.linalg.spsolve(A,B,use_umfpack=True)
-                 else: 
-                  X = (lu[(m,n)] if (m,n) in lu.keys() else lu.setdefault((m,n),sp.linalg.splu(A))).solve(B)
-
-                 kappap[m,q] = np.dot(mesh['kappa_mask'],X)#-DeltaT)
-
+                 X = solve(argv,A,B,(q,m))
+                 kappap[m,q] = np.dot(mesh['kappa_mask'],X)
+                 #-------------------------
+ 
                  if argv['multiscale']:
-                  error = abs(kappap[m,q] - kappa_fourier_m[m])/abs(kappap[m,q])
-                  if error < argv['multiscale_error'] and m <= transition[q]:
-
-                   fourier = True   
-                   if argv.setdefault('transition',False):
-                       transitionp[q] = m
-                   else:
-                       transitionp[q] = 1e4
+                  error = abs(kappap[m,q] - kappap_f[m,q])/abs(kappap[m,q])
+                  if error < argv['multiscale_error_fourier'] and m <= transition[q]:
 
+                   transitionp[q] = m  if argv.setdefault('transition',False) else 1e4
+   
                    #Vectorize
-                   #kappap[:m+1,q] = kappa_fourier_m[:m+1]
-                   kappap[:m,q] = kappa_fourier_m[:m]
-                   diffusive += m
-                   #Xm = tf[:m+1] - np.einsum('m,i,mci->mc',mat['mfp_sampled'][:m+1],mat['VMFP'][q,:argv['dim']],tfg[:m+1])
-                   Xm = tf[:m] - np.einsum('m,i,mci->mc',mat['mfp_sampled'][:m],mat['VMFP'][q,:argv['dim']],tfg[:m])
-                   #DeltaTp += np.einsum('mc,m->c',Xm,mat['tc'][:m+1,q])  
-                   DeltaTp += np.einsum('mc,m->c',Xm,mat['tc'][:m,q])  
+                   kappap[:m+1,q] = kappap_f[:m+1,q]
+                   diffusive += m+1
+                   Xm = tf[:m+1] - np.einsum('m,i,mci->mc',mat['mfp_sampled'][:m+1],mat['VMFP'][q,:argv['dim']],tfg[:m+1])
+
+                   DeltaTp += np.einsum('mc,m->c',Xm,mat['tc'][:m+1,q])  
                   
-                   #np.add.at(TBp,np.arange(mesh['eb'].shape[0]),-np.einsum('mc,mc->c',Xm[:,mesh['eb']],GG[:m+1,q]))
-                  
-                   
                    if len(mesh['eb']) > 0:
-                    np.add.at(TBp,np.arange(mesh['eb'].shape[0]),-np.einsum('mc,mc->c',Xm[:,mesh['eb']],GG[:m,q]))
-                   #Jp += np.einsum('mc,mj->cj',Xm,sigma[:m+1,q,0:argv['dim']])*1e-18
-                   Jp += np.einsum('mc,mj->cj',Xm,sigma[:m,q,0:argv['dim']])*1e-18
+                    np.add.at(TBp,np.arange(mesh['eb'].shape[0]),-np.einsum('mc,mc->c',Xm[:,mesh['eb']],GG[:m+1,q]))
+                   
+                   Jp += np.einsum('mc,mj->cj',Xm,sigma[:m+1,q,0:argv['dim']])*1e-18
                    break
 
                  DeltaTp += X*mat['tc'][m,q]
-                #for c,i in enumerate(mesh['eb']): TBp[m,c] -= X[i]*GG[m,q,c]
 
                  if len(mesh['eb']) > 0:
                   np.add.at(TBp,np.arange(mesh['eb'].shape[0]),-X[mesh['eb']]*GG[m,q])
 
                  Jp += np.einsum('c,j->cj',X,sigma[m,q,0:argv['dim']])*1e-18
-                 #Supp += kappap[m,q]*mat['suppression'][m,q,:]*1e9
                 
  
-           for m in range(argv['n_serial'])[idx[0]+1:]:
-
+           for m in range(argv['n_serial'])[idx+1:]:
 
+               #----------------------------------------------------
                B = DeltaT +  mfp[m]*(P[n] +get_boundary(RHS[n],mesh['eb'],n_elems))
                A = get_m(Master,np.concatenate((mfp[m]*Gm[n],mfp[m]*D[n]+np.ones(n_elems)))[conversion])
-               if not argv['keep_lu']:
-                  X = sp.linalg.spsolve(A,B,use_umfpack=True)
-               else: 
-                  X = (lu[(m,n)] if (m,n) in lu.keys() else lu.setdefault((m,n),sp.linalg.splu(A))).solve(B)
-
+               X = solve(argv,A,B,(q,m))
                kappap[m,q] = np.dot(mesh['kappa_mask'],X)
+               #-------------------------------------------------------
 
-               error_bal = abs(kappap[m,q] - kappa_bal)/abs(kappap[m,q])
-               if error_bal < argv['multiscale_error'] and \
-                  abs(kappap[m-1,q] - kappa_bal)/abs(kappap[m-1,q]) < argv['multiscale_error'] and  m > int(len(mat['mfp_sampled'])/2):
+               error_bal = abs(kappap[m,q] - kappap_b[m,q])/abs(kappap[m,q])
+               if error_bal < argv['multiscale_error_ballistic'] and \
+                   abs(kappap[m-1,q] - kappap_b[m-1,q])/abs(kappap[m-1,q]) < argv['multiscale_error_ballistic'] and  m > int(len(mat['mfp_sampled'])/2):
 
-                   kappap[m:,q] = kappa_bal
+                   kappap[m:,q] = kappap_b[m:,q]
                    bal += argv['n_serial']-m
                    DeltaTp += X_bal*np.sum(mat['tc'][m:,q])
-                   #for c,i in enumerate(mesh['eb']): TBp[m:,c] -= X_bal[i]*GG[m:,q,c]
 
-                   np.add.at(TBp,np.arange(mesh['eb'].shape[0]),-np.einsum('c,mc->c',X_bal[mesh['eb']],GG[m:,q]))
+                   if len(mesh['eb']) > 0:
+                    np.add.at(TBp,np.arange(mesh['eb'].shape[0]),-np.einsum('c,mc->c',X_bal[mesh['eb']],GG[m:,q]))
 
                    Jp += np.einsum('c,j->cj',X_bal,np.sum(sigma[m:,q,0:argv['dim']],axis=0))*1e-18
-                   #Supp += np.einsum('m,mu->u',kappap[m:,q],mat['suppression'][m:,q,:])*1e9
                    break
 
                DeltaTp += X*mat['tc'][m,q]
                
-               #for c,i in enumerate(mesh['eb']): TBp[m,c] -= X[i]*GG[m,q,c]
-               #np.add.at(TBp[m],np.arange(mesh['eb'].shape[0]),-X[mesh['eb']]*GG[m,q])
                if len(mesh['eb'])>0:
                 np.add.at(TBp,np.arange(mesh['eb'].shape[0]),-X[mesh['eb']]*GG[m,q])
 
                Jp += np.einsum('c,j->cj',X,sigma[m,q,0:argv['dim']])*1e-18
-               #Supp += kappap[m,q]*mat['suppression'][m,q,:]*1e9
           
 
         Mp[0] = diffusive
         Mp[1] = bal
+
+        #Centering--
+        #DeltaT = DeltaT - (max(DeltaT)+min(DeltaT))/2.0
+
+        DeltaT_old = DeltaT.copy()
+
         comm.Barrier()
 
         comm.Allreduce([DeltaTp,MPI.DOUBLE],[DeltaT,MPI.DOUBLE],op=MPI.SUM)
         comm.Allreduce([Jp,MPI.DOUBLE],[J,MPI.DOUBLE],op=MPI.SUM)
         comm.Allreduce([kappap,MPI.DOUBLE],[kappa,MPI.DOUBLE],op=MPI.SUM)
+        comm.Allreduce([kappap_b,MPI.DOUBLE],[kappab,MPI.DOUBLE],op=MPI.SUM)
+        comm.Allreduce([kappap_0,MPI.DOUBLE],[kappa0,MPI.DOUBLE],op=MPI.SUM)
+        comm.Allreduce([kappap_f,MPI.DOUBLE],[kappaf,MPI.DOUBLE],op=MPI.SUM)
         comm.Allreduce([Mp,MPI.DOUBLE],[MM,MPI.DOUBLE],op=MPI.SUM)
         comm.Allreduce([transitionp,MPI.DOUBLE],[transition,MPI.DOUBLE],op=MPI.SUM)
         comm.Allreduce([TBp,MPI.DOUBLE],[TB,MPI.DOUBLE],op=MPI.SUM)
-        #comm.Allreduce([Supp,MPI.DOUBLE],[Sup,MPI.DOUBLE],op=MPI.SUM)
-        #comm.Allreduce([Supdp,MPI.DOUBLE],[Supd,MPI.DOUBLE],op=MPI.SUM)
-        #comm.Allreduce([Supbp,MPI.DOUBLE],[Supb,MPI.DOUBLE],op=MPI.SUM)
         kappa_totp = np.array([np.einsum('mq,mq->',sigma[:,argv['rr'],int(mesh['meta'][-1])],kappa[:,argv['rr']])])
+        kappaf_totp = np.array([np.einsum('mq,mq->',sigma[:,argv['rr'],int(mesh['meta'][-1])],kappaf[:,argv['rr']])])
         comm.Allreduce([kappa_totp,MPI.DOUBLE],[kappa_tot,MPI.DOUBLE],op=MPI.SUM)
-
+        comm.Allreduce([kappaf_totp,MPI.DOUBLE],[kappaf_tot,MPI.DOUBLE],op=MPI.SUM)
 
         if argv['multiscale'] and comm.rank == 0: print_multiscale(argv,MM) 
 
@@ -293,24 +286,24 @@ def solve_rta(**argv):
         if argv['verbose'] and comm.rank == 0:   
          print('{0:8d} {1:24.4E} {2:22.4E}'.format(kk,kappa_vec[-1],error),flush=True)
 
+
     if argv['verbose'] and comm.rank == 0:
       print(colored(' -----------------------------------------------------------','green'),flush=True)
 
 
-    #print(np.dot(mesh['kappa_mask'],DeltaT)*1e18)
-    output =  {'kappa':kappa_vec,'temperature':DeltaT,'flux':J,'kappa_mode':kappa-np.dot(mesh['kappa_mask'],DeltaT)}#,'suppression':Sup}
-    #output =  {'kappa':kappa_vec,'temperature':DeltaT,'flux':J,'kappa_mode':kappa}#,'suppression':Sup}
-    #if argv['multiscale']:   
-    #    output.update({'suppression_diffusive':Supd})
-    #    output.update({'suppression_diffusive':Supd,'suppression_ballistic':Supb})
+
+    kappaf -=np.dot(mesh['kappa_mask'],DeltaT_old)
+    kappa -=np.dot(mesh['kappa_mask'],DeltaT_old)
+    output =  {'kappa':kappa_vec,'temperature':DeltaT,'flux':J,'kappa_mode':kappa,'kappa_mode_f':kappaf,\
+                'kappa_mode_b':kappab-np.dot(mesh['kappa_mask'],DeltaT_old)}
+
+
 
     return output
 
 
 
           
-
-
      
 
 
