@@ -15,6 +15,7 @@ from .solve_mfp import *
 from .solve_rta import *
 from .solve_full import *
 import pkg_resources  
+from .fourier import *
 
 comm = MPI.COMM_WORLD
 
@@ -28,7 +29,8 @@ class Solver(object):
         self.state = {}
         self.multiscale = argv.setdefault('multiscale',False)
         #self.umfpack = argv.setdefault('umfpack',False)
-        self.error_multiscale = argv.setdefault('multiscale_error',1e-2)
+        self.error_multiscale_fourier = argv.setdefault('multiscale_error_fourier',5e-2)
+        self.error_multiscale_ballistic = argv.setdefault('multiscale_error_ballistic',1e-2)
         self.bundle = argv.setdefault('bundle',False)
         self.verbose = argv.setdefault('verbose',True)
 
@@ -38,8 +40,8 @@ class Solver(object):
         self.relaxation_factor = argv.setdefault('alpha',1.0)
         self.keep_lu = argv.setdefault('keep_lu',True)
         self.only_fourier = argv.setdefault('only_fourier',False)
-        self.max_bte_iter = argv.setdefault('max_bte_iter',10)
-        self.min_bte_iter = argv.setdefault('min_bte_iter',20)
+        self.max_bte_iter = argv.setdefault('max_bte_iter',20)
+        #self.min_bte_iter = argv.setdefault('min_bte_iter',20)
         self.max_bte_error = argv.setdefault('max_bte_error',1e-3)
         self.max_fourier_iter = argv.setdefault('max_fourier_iter',20)
         self.deviational = argv.setdefault('deviational',False)
@@ -58,7 +60,7 @@ class Solver(object):
         #-----IMPORT MESH--------------------------------------------------------------
         if comm.rank == 0:
          
-         data = argv['geometry'].data if 'geometry' in argv.keys() else load_data('geometry')
+         data = argv['geometry'] if 'geometry' in argv.keys() else load_data('geometry')
 
 
          self.n_elems = int(data['meta'][0])
@@ -93,28 +95,27 @@ class Solver(object):
           #data = argv['material'].data if 'material' in argv.keys() else dd.io.load(value)
 
           if 'material' in argv.keys():
-            data = argv['material'].data
+            data = argv['material']
           else:            
             data = load_data(value)  
-        
-          
-          #data['VMFP']  *= 1e9
-          #data['sigma'] *= 1e9
+       
           data['kappa'] = data['kappa'][0:self.dim,0:self.dim] 
           if data['model'][0] == 10 : #full
            data['B'] = np.einsum('i,ij->ij',data['scale'],data['B'].T + data['B'])
          else: data = None
-         #self.__dict__.update(create_shared_memory_dict(data))
+
          self.mat.update({key:create_shared_memory_dict(data)})
          self.set_model(self.mat[key]['model'][0])
         argv['mat'] = self.mat[0]
         #----------------------
 
-        #Build elem_kappa_map
+        #Build elem_kappa_mapi
         self.elem_kappa_map = np.array([ list(self.mat[i]['kappa'])  for i in self.mesh['elem_mat_map']])
-        #self.elem_kappa_map = np.array([ list(np.eye(2)*self.mat[i]['kappa'][0,0])  for i in self.mesh['elem_mat_map']])
+        self.mesh['elem_kappa_map'] = self.elem_kappa_map
 
 
+        #self.elem_kappa_map = np.array([ list(np.eye(2)*self.mat[i]['kappa'][0,0])  for i in self.mesh['elem_mat_map']])
+
         if comm.rank == 0 and self.verbose: self.bulk_info()
 
         if self.model[0:3] == 'mfp' or self.model[0:3] == 'rta' or self.model[0:3] == 'Gra':
@@ -124,6 +125,9 @@ class Solver(object):
          self.ff = range(block*comm.rank,self.n_serial) if comm.rank == comm.size-1 else range(block*comm.rank,block*(comm.rank+1))
          if comm.rank == 0 and self.verbose: self.mfp_info()
 
+        elif self.model == 'Fourier':
+          argv['only_fourier'] = True 
+          self.only_fourier = True
 
         elif self.model == 'full': 
          self.n_parallel = len(self.mat[0]['tc'])
@@ -136,9 +140,10 @@ class Solver(object):
                 print(" ",flush=True)
 
         if comm.rank == 0:
-          data = self.solve_fourier(self.elem_kappa_map)
+          argv['cache'] = {}  
+          data = solve_fourier_single(self.elem_kappa_map,**argv)
+
 
-          
           variables = {0:{'name':'Temperature Fourier','units':'K','data':data['temperature_fourier'],'increment':-self.mesh['applied_gradient']},\
                        1:{'name':'Flux Fourier'       ,'units':'W/m/m','data':data['flux_fourier'],'increment':[0,0,0]}}
 
@@ -163,29 +168,22 @@ class Solver(object):
          argv['kappa_factor'] = self.kappa_factor
          
          if argv.setdefault('user',False):
+
             argv['n_serial'] = self.n_serial
-            data = argv['solve'](**argv)
-            #data = solve_interface(**argv)
+
+            data = argv['user_model'].solve(**argv)
 
          elif self.model[0:3] == 'mfp' or \
             self.model[0:3] == 'Gra' :
             argv['n_serial'] = self.n_serial
-            #a = time.time()
-            #data = self.solve_mfp(**argv) 
-            #quit()
-            #a = time.time()
-            #if argv.setdefault('deviational',False):
-            # data = solve_deviational(**argv) 
-            #else: 
             data = solve_mfp(**argv) 
-            #print(time.time()-a)
+
          elif self.model[0:3] == 'rta':
             argv['n_serial'] = self.n_serial
             data = solve_rta(**argv) 
-            #data = self.solve_rta(**argv)
+
          elif self.model == 'full':
             data = solve_full(**argv) 
-            #data = self.solve_full(**argv)
 
 
          #self.state.update({'kappa':data['kappa_vec']})
@@ -243,15 +241,16 @@ class Solver(object):
           print(colored('  Relaxation:                              ','green')+ str(self.relaxation_factor),flush=True)
           print(colored('  Keep Lu:                                 ','green')+ str(self.keep_lu),flush=True)
           #print(colored('  Use umfpack                              ','green')+ str(self.umfpack),flush=True)
-          print(colored('  Deviational                              ','green')+ str(self.deviational),flush=True)
+          #print(colored('  Deviational                              ','green')+ str(self.deviational),flush=True)
           print(colored('  Load State                               ','green')+ str(self.load_state),flush=True)
           print(colored('  Save State                               ','green')+ str(self.save_state),flush=True)
-          print(colored('  Multiscale Error:                        ','green')+ str(self.error_multiscale),flush=True)
+          print(colored('  Multiscale Error (Fourier):              ','green')+ str(self.error_multiscale_fourier),flush=True)
+          print(colored('  Multiscale Error (Ballistic):            ','green')+ str(self.error_multiscale_ballistic),flush=True)
           print(colored('  Only Fourier:                            ','green')+ str(self.only_fourier),flush=True)
           print(colored('  Max Fourier Error:                       ','green')+ '%.1E' % (self.max_fourier_error),flush=True)
           print(colored('  Max Fourier Iter:                        ','green')+ str(self.max_fourier_iter),flush=True)
           print(colored('  Max BTE Error:                           ','green')+ '%.1E' % (self.max_bte_error),flush=True)
-          print(colored('  Min BTE Iter:                            ','green')+ str(self.min_bte_iter),flush=True)
+          #print(colored('  Min BTE Iter:                            ','green')+ str(self.min_bte_iter),flush=True)
           print(colored('  Max BTE Iter:                            ','green')+ str(self.max_bte_iter),flush=True)
 
   def full_info(self):
@@ -350,190 +349,6 @@ class Solver(object):
 
 
 
-  def get_decomposed_directions(self,ll,rot):
-
-    normal = self.mesh['face_normals'][ll,0:self.dim]
-    dist   = self.mesh['dists'][ll,0:self.dim]
-    v_orth = np.dot(normal,np.dot(rot,normal))/np.dot(normal,dist)
-    v_non_orth = np.dot(rot,normal) - dist*v_orth
-
-    return v_orth,v_non_orth[:self.dim]
-
-  def get_kappa(self,i,j,ll,kappa):
-
-   if i ==j:
-    return np.array(kappa[i])
-   
-   normal = self.mesh['face_normals'][ll,0:self.dim]
-
-   kappa_i = np.array(kappa[i])
-   kappa_j = np.array(kappa[j])
-
-   ki = np.dot(normal,np.dot(kappa_i,normal))
-   kj = np.dot(normal,np.dot(kappa_j,normal))
-   w  = self.mesh['interp_weigths'][ll]
-
-   kappa_loc = kj*kappa_i/(ki*(1-w) + kj*w)
-
-   return kappa_loc
-
-  
-
-  def compute_secondary_flux(self,temp,kappa):
-
-   #if not np.isscalar(kappa):
-   #  kappa = kappa[0,0]
-
-   diff_temp = []
-   for i in self.mesh['side_per_elem']:
-       diff_temp.append(i*[0])
-
-   a = time.time()
-   for ll in self.mesh['active_sides'] :
-    elems = self.mesh['side_elem_map_vec'][ll]
-    kc1 = elems[0]
-    c1 = self.mesh['centroids'][kc1]
-
-    ind1 = list(self.mesh['elem_side_map_vec'][kc1]).index(ll)
-    if not ll in self.mesh['boundary_sides']: 
-     kc2 = elems[1]
-     ind2 = list(self.mesh['elem_side_map_vec'][kc2]).index(ll)
-     temp_1 = temp[kc1]
-     temp_2 = temp[kc2]
-
-     if ll in self.mesh['periodic_sides']:
-      temp_2 += self.mesh['periodic_side_values'][list(self.mesh['periodic_sides']).index(ll)]
-
-     diff_t = temp_2 - temp_1
-     diff_temp[kc1][ind1]  = diff_t
-     diff_temp[kc2][ind2]  = -diff_t
-
-   gradT = np.zeros((self.n_elems,self.dim))
-   for k,dt in enumerate(diff_temp):
-    gradT[k] = np.einsum('js,s->j',self.mesh['weigths'][k,:,:self.mesh['side_per_elem'][k]],np.array(dt))
-
-   #-----------------------------------------------------
-   F_ave = np.zeros((len(self.mesh['sides']),self.dim))
-   for ll in self.mesh['active_sides']:
-      (i,j) = self.mesh['side_elem_map_vec'][ll]
-      if not i==j:
-       kappa_loc = self.get_kappa(i,j,ll,kappa)
-       w = self.mesh['interp_weigths'][ll]
-       F_ave[ll] = w*gradT[i] + (1.0-w)*gradT[j]
-    
-
-   C = np.zeros(self.n_elems)
-   for ll in self.mesh['active_sides']:
-    (i,j) = self.mesh['side_elem_map_vec'][ll]
-    if not i==j:
-      kappa_loc = self.get_kappa(i,j,ll,kappa)
-      area = self.mesh['areas'][ll]   
-      (dummy,v_non_orth) = self.get_decomposed_directions(ll,rot=kappa_loc)
-      tmp = np.dot(F_ave[ll],v_non_orth)*area
-      C[i] += tmp/self.mesh['volumes'][i]
-      C[j] -= tmp/self.mesh['volumes'][j]
-
-
-   return C,gradT
-
-
-  def solve_fourier(self,kappa,**argv):
-
-   if np.isscalar(kappa):
-       kappa = np.diag(np.diag(kappa*np.eye(self.dim)))
-
-   if kappa.ndim == 2:
-      kappa = np.repeat(np.array([np.diag(np.diag(kappa))]),self.n_elems,axis=0)
-
-   F = sp.dok_matrix((self.n_elems,self.n_elems))
-   B = np.zeros(self.n_elems)
-   for ll in self.mesh['active_sides']:
-
-      area = self.mesh['areas'][ll] 
-      (i,j) = self.mesh['side_elem_map_vec'][ll]
-      vi = self.mesh['volumes'][i]
-      vj = self.mesh['volumes'][j]
-      kappa_loc = self.get_kappa(i,j,ll,kappa)
-      #kappa_loc = np.eye(2)*kappa_loc[0,0] 
-
-      if not i == j:
-
-       (v_orth,dummy) = self.get_decomposed_directions(ll,rot=kappa_loc)
-       F[i,i] += v_orth/vi*area
-       F[i,j] -= v_orth/vi*area
-       F[j,j] += v_orth/vj*area
-       F[j,i] -= v_orth/vj*area
-       if ll in self.mesh['periodic_sides']:    
-        kk = list(self.mesh['periodic_sides']).index(ll)   
-        B[i] += self.mesh['periodic_side_values'][kk]*v_orth/vi*area
-        B[j] -= self.mesh['periodic_side_values'][kk]*v_orth/vj*area
-   
-    
-   ##rescaleand fix one point to 0
-   #scale = 1/F.max(axis=0).toarray()[0]
-   #n = np.random.randint(self.n_elems)
-   #scale[n] = 0
-   #F.data = F.data * scale[F.indices]
-   #F[n,n] = 1
-   #B[n] = 0
-   SU = splu(F.tocsc())
-    #-----------------------
-
-   C = np.zeros(self.n_elems)
-   n_iter = 0
-   kappa_old = 0
-   error = 1  
-   grad = np.zeros((self.n_elems,self.dim))
-   while error > self.max_fourier_error and \
-                  n_iter < self.max_fourier_iter :
-        RHS = B + C
-        #for n in range(self.n_elems):
-        #  RHS[n] = RHS[n]*scale[n]  
-
-        temp = SU.solve(RHS)
-        temp = temp - (max(temp)+min(temp))/2.0
-        kappa_eff = self.compute_diffusive_thermal_conductivity(temp,grad,kappa)
-        error = abs((kappa_eff - kappa_old)/kappa_eff)
-        kappa_old = kappa_eff
-        n_iter +=1
-
-        C,grad = self.compute_secondary_flux(temp,kappa)
-
-
-   flux = -np.einsum('cij,cj->ci',kappa,grad)
- 
-   meta = [kappa_eff,error,n_iter] 
-   return {'flux_fourier':flux,'temperature_fourier':temp,'meta':np.array(meta),'grad':grad}
-
-
-
-
-  def compute_diffusive_thermal_conductivity(self,temp,gradT,kappa):
-
-   kappa_eff = 0
-   for l in self.mesh['flux_sides']:
-
-    (i,j) = self.mesh['side_elem_map_vec'][l]
-
-    
-    if np.isscalar(kappa):
-      kappa = np.diag(np.diag(kappa*np.eye(self.dim)))
- 
-    if kappa.ndim == 3:
-      kappa = self.get_kappa(i,j,l,kappa)
-      
-    (v_orth,v_non_orth) = self.get_decomposed_directions(l,rot=kappa)
-
-    deltaT = temp[i] - (temp[j] + 1)
-    kappa_eff -= v_orth *  deltaT * self.mesh['areas'][l]
-    w  = self.mesh['interp_weigths'][l]
-    grad_ave = w*gradT[i] + (1.0-w)*gradT[j]
-
-    kappa_eff += np.dot(grad_ave,v_non_orth)/2 * self.mesh['areas'][l]
-
-
-   return kappa_eff*self.kappa_factor
-
   def print_logo(self):
 
 
@@ -549,11 +364,11 @@ class Solver(object):
     print(flush=True)
     print('                       GENERAL INFO',flush=True)
     print(colored(' -----------------------------------------------------------','green'),flush=True)
-    print(colored('  Contact:          ','green') + 'romanog@mit.edu                       ',flush=True) 
-    print(colored('  Source code:      ','green') + 'https://github.com/romanodev/OpenBTE  ',flush=True)
-    print(colored('  Become a sponsor: ','green') + 'https://github.com/sponsors/romanodev ',flush=True)
-    print(colored('  Cloud:            ','green') + 'https://shorturl.at/cwDIP             ',flush=True)
-    print(colored('  Mailing List:     ','green') + 'https://shorturl.at/admB0             ',flush=True)
+    print(colored('  Contact:          ','green') + 'romanog@mit.edu                        ',flush=True) 
+    print(colored('  Source code:      ','green') + 'https://github.com/romanodev/OpenBTE     ',flush=True)
+    print(colored('  Become a sponsor: ','green') + 'https://github.com/sponsors/romanodev    ',flush=True)
+    print(colored('  Documentation:    ','green') + 'https://openbte.readthedocs.io',flush=True)
+    print(colored(' -----------------------------------------------------------','green'),flush=True)
+    print('                       WARNINGS',flush=True)
     print(colored(' -----------------------------------------------------------','green'),flush=True)
     print(flush=True)   
-
