@@ -4,7 +4,7 @@ import plotly.graph_objs as go
 import numpy as np
 import sys
 from .utils import *
-
+import time
 
 
 def plotly_trisurf(nodes, simplices, data,name,units,visible=False):
@@ -35,12 +35,17 @@ def add_data(nodes,elems,name,data,units,buttons,fig,nv):
       buttons.append(dict(label=name,
                      method="update",
                      args=[{"visible":visible},]))
+      return  plotly_data
 
 
-def plot_results(data,nodes,elems,**argv):
+def plot_results(master,**argv):
 
    
+   data  = master['variables']
+   nodes = master['nodes']
+   elems = master['elems']
 
+  
    size = [max(nodes[:,i]) - min(nodes[:,i])  for i in range(3)] 
    dim = 2   if size[2] == 0 else 3
 
@@ -55,16 +60,22 @@ def plot_results(data,nodes,elems,**argv):
    buttons = []
    fig = go.Figure()
    current = 0
+   #plotly_tot_tmp = {}
+  
    for key,value in data.items():
-
      if value['data'].ndim == 1: #scalar
-      add_data(nodes,elems,value['name'],value['data'],value['units'],buttons,fig,nv)
+      plotly_data = add_data(nodes,elems,value['name'],value['data'],value['units'],buttons,fig,nv)
      elif value['data'].ndim == 2 : #vector 
-         add_data(nodes,elems,value['name'] + '(x)',value['data'][:,0],value['units'],buttons,fig,nv)
-         add_data(nodes,elems,value['name'] + '(y)',value['data'][:,1],value['units'],buttons,fig,nv)
-         if dim == 3: add_data(nodes,elems,value['name']+ '(z)',value['data'][:,2],value['units'],buttons,fig,nv)
+         plotly_data = add_data(nodes,elems,value['name'] + '(x)',value['data'][:,0],value['units'],buttons,fig,nv)
+         plotly_data = add_data(nodes,elems,value['name'] + '(y)',value['data'][:,1],value['units'],buttons,fig,nv)
+         if dim == 3: 
+             plotly_data = add_data(nodes,elems,value['name']+ '(z)',value['data'][:,2],value['units'],buttons,fig,nv)
+
          mag = [np.linalg.norm(value) for value in value['data']]
-         add_data(nodes,elems,value['name'] + '(mag.)',mag,value['units'],buttons,fig,nv)
+         plotly_data = add_data(nodes,elems,value['name'] + '(mag.)',mag,value['units'],buttons,fig,nv)
+
+   
+   #-----------------------
 
    fig.update_layout(
     font=dict(
@@ -83,11 +94,18 @@ def plot_results(data,nodes,elems,**argv):
         'xanchor': 'center',
         'yanchor': 'top'})
 
-   fig.update_layout(width=400,height=450,autosize=True,margin=dict(t=20, b=10, l=00, r=20),paper_bgcolor='rgba(0,0,0,0)',plot_bgcolor='rgba(0,0,0,0)')  
+
+   if argv.setdefault('large',False):
+       x1 = 700
+       x2 = 500
+   else:    
+       x1 = 400
+       x2 = 450
+
+   fig.update_layout(width=x1,height=x2,autosize=True,margin=dict(t=20, b=10, l=00, r=20),paper_bgcolor='rgba(0,0,0,0)',plot_bgcolor='rgba(0,0,0,0)')  
 
 
    updatemenus=[dict(direction='down',active=nv-5 if nv > 1 else 0,x=0.4,y=1.15,buttons=list(buttons),showactive=True)]
-   fig.update_layout(updatemenus=updatemenus)
 
 
    #update axes---------------
@@ -136,34 +154,24 @@ def plot_results(data,nodes,elems,**argv):
    fig.update_layout(xaxis_showgrid=False, yaxis_showgrid=False)
    fig.update_layout(scene_camera=camera)
 
+   fig.update_layout(updatemenus=updatemenus)
 
    if argv.setdefault('write_html',False):
+   
     fig.write_html("plotly.html")
 
 
    if argv.setdefault('write_data',False):
-       idd = 0
-       data_rolled = {} 
-       for key,value in data.items():
-        if value['data'].ndim == 1: #scalar
-            data_rolled.update({idd:{'name':value['name'],'data':value['data'],'units':value['units']}}); idd +=1
-        elif value['data'].ndim == 2 : #vector 
-            data_rolled.update({idd:{'name':value['name'] + ' (x)','data':value['data'][:,0],'units':value['units']}}); idd+=1
-            data_rolled.update({idd:{'name':value['name'] + ' (y)','data':value['data'][:,1],'units':value['units']}}); idd+=1
-            if dim == 3: 
-                data_rolled.update({idd:{'name':value['name'] + ' (z)','data':value['data'][:,2],'units':value['units']}}); idd+=1
-            mag = [np.linalg.norm(value) for value in value['data']]
-            data_rolled.update({idd:{'name':value['name'] + ' (mag)','data':mag,'units':value['units']}}); idd +=1
 
-       tmp = {'elems':elems,'nodes':nodes,'variables':data_rolled,'fourier':argv['fourier'],'bulk':argv['bulk']}
+       variables = {}
+       for d in fig.data:
+        variables[d['name']]=d
 
-       if 'bte' in argv.keys():
-           tmp['bte'] = argv['bulk']
-
-
-       data = {argv.setdefault('name','output'):tmp}
-       save_data(argv.setdefault('name','output'),data)
+       plotly_tot = {'sample_0':{'variables':variables,'size':size}}
+      
+       save_data('output_new',plotly_tot)
 
+       #plotly.io.write_json(fig,'ff.json')
 
    # plotly.io.to_image(fig,format='png')
    #fig.write_image("test.png",scale=5)
@@ -173,10 +181,6 @@ def plot_results(data,nodes,elems,**argv):
     else:
      fig.show(renderer='browser')
 
-   #if argv.setdefault('show',False):
-   #else:
-   #app = dash.Dash(__name__, external_stylesheets=['https://codepen.io/chriddyp/pen/bWLwgP.css'])
-   #app.layout = html.Div(children=[dcc.Graph(figure=fig)])
-   #app.run_server()
 
+   return fig
 
